/*
 * This file is part of the R&S 1MA226 application note.
 *
 * Copyright (C) 2016 Rohde & Schwarz <www.rohde-schwarz.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 */

// OSP remote path control
// software tool for OSP switching via network analyzers
// according to application note 1MA_226
// Author: Dieter Bues,  
// Project: c:\Dev-Cpp\Examples\1MA226_SetPath\SetPath.dev
// Development system: c:\W\DEV_CPP\Dev-Cpp 5.11 TDM-GCC 4.9.2 Setup.exe

#include <cstdlib>
#include <iostream>
#include <string>

#include <time.h>

#include "visa.h"
#define LargeBuffLen 200000
#define SmallBuffLen 2048
#define MaxStringSize 300
#define DEBUG_ON 128

#pragma warning(disable : 4996)

#define version "OSP remote path switching, v. 1.1, R&S 1MAA_DB, July 12th, 2016"

// Revision history:
// listed from top to bottom, i.e. the actual one is at the end of the list
    
// First release: Aug. 30th, 2013	
	        
// (1) Show the *IDN? result in debug mode
//     The result so far was not shown
//     implemented in parallel with FSV implementation into 1MA226
//     Appropriate version message of LoadStat:  1.1
//     Test parameter : (to be set in DEV-CPP / Execute / Parameter)
//     TCPIP::10.85.0.36::INSTR pt_02 128
//  Tag: [db_000]
//  Date:  June 8th, 2016

            
// (2) Show the version message if no parameters are specified
//     When the program is invoked w.o. parameters, the version should
//     be at least displayed
//  Tag: [db_001]
//  Date:  June 10th, 2016

using namespace std;

 ViSession instr;
 ViSession defaultRM;
 ViStatus status;
 ViUInt32 retCount;
 ViUInt32 writeCount;
 unsigned char buffer[SmallBuffLen];
 char stringinput[SmallBuffLen];

 bool debugx = false;
//const bool debugx = false;

enum {
  status_OK,
  status_INVALID_NUMBER_OF_PARAMETERS,
  status_REM_IO_ERR,
  status_REM_OPEN_ERR,
  status_INVALID_ADDRESS,
  status_UNKNOWN_ERR,
  status_INVALID_PATH
};

string par_1("TCPIP::10.110.10.253::INSTR");
string par_2("ROUT:CLOS ");
string par_3 =("1");

unsigned int status_code = status_UNKNOWN_ERR;   // suppose NOK


// standard remote open function 
ViSession RemOpen (string remote_address)
{
         ViSession instr;
         //ViSession defaultRM;
         ViStatus status;
         char addr_buff[100];
         
         int x,i;

        x=remote_address.find("INSTR");
        
        if (x>0)
        {
            for(i=0;i< x+5;i++)
                addr_buff[i]= remote_address[i];
            addr_buff[i]=0;
                
                status=viOpenDefaultRM (&defaultRM);
                if (status < VI_SUCCESS)
                  status_code = status_REM_OPEN_ERR;
                 
                //status = viOpen (defaultRM, (ViRsrc)  "TCPIP::10.110.10.44::INSTR", VI_NULL, VI_NULL, &instr);
                //
                status = viOpen (defaultRM, (ViRsrc) addr_buff, VI_NULL, VI_NULL, &instr);
                status = viSetAttribute (instr, VI_ATTR_TMO_VALUE, 500);
                return(instr);
        }
        status_code = status_INVALID_ADDRESS;
		return instr;
}

// standard remote output function 
void RemOut (ViSession instr,string inpar)
{
    ViUInt32 writeCount;
    ViStatus status;
    char stringinput[512];
    unsigned int i;
    unsigned int x;
    
    for(i=0;i<inpar.length();i++)
        stringinput[i]=inpar[i];
    
    stringinput[i]=0;

    x=inpar.length();

    status = viWrite (instr, (ViBuf)stringinput, (ViUInt32) x, &writeCount);
    if (status < VI_SUCCESS)
       status_code = status_REM_IO_ERR;
}



// standard remote input function
// for standard SCPI communication only
string RemInp (ViSession instr)
{
   ViUInt32 retCount;
   ViStatus status;
   unsigned char buffer[SmallBuffLen];
   //unsigned char buffer[LargeBuffLen];
   //string xx="                                                                     ";
   string xx (MaxStringSize, ' ');
   int i;
   
   status = viRead (instr, buffer, sizeof buffer, &retCount);

   if (status < VI_SUCCESS)
    {
       status_code = status_REM_IO_ERR;
    }

    for(i=0 ; i<(int)retCount && i<SmallBuffLen ; i++)
        xx[i]=buffer[i];
    xx[i]=0;

    return(xx);

}



// standard remote close function
void RemClose (ViSession instr)
{
   ViStatus status;         
   
   status = viClose(instr);
   status = viClose(defaultRM);
}


// replacement for stoi which is available
// in newer compiler versions
int xstoi (string inpar)
{
  int i=0;
  int x;
  char buff[10];
  
  while((inpar[i]!=0) && (i<10))
  {
    buff[i]=inpar[i++];
  }
  buff[++i]=0;   // set string delimiter
  x=atoi(buff);
  return(x);
}


// ************************ main program **************************
int main(int argc, char *argv[])
{
    int x;
	unsigned int i;
    string result;
    string xx,yy;
    int stat_STB, stat_ESR;
    long byte_count,n;
    char dat_buff[LargeBuffLen];
    ViSession OSP;

	if (argc > 3)
	{
		int debugInt = atoi(argv[3]);
		if((debugInt & DEBUG_ON) == DEBUG_ON)
			debugx = true;
	}

    // [db_001]
	if (argc == 1)
	  debugx = true;
	
	
    if (debugx)
     {
       cout << version  << endl;
       cout << endl;
       }
    
    if (debugx)
      {
       cout << "**1** number of command line parameters:   ";         
       cout << argc -1 << endl;    // decreased by 1 because program name doesn't count
      }

     if (debugx)
     {
      cout << endl << "**2** commands detected :" << endl;
      for(i=1;i<argc;i++)
        {
          cout << i << ": ";                 
          cout << argv[i] << endl;
         }
      cout <<  endl;
      }

 
     if ((argc==3) || (argc==4) )
     {
       par_1=argv[1];   // remote control address
       par_2=par_2+"\""+argv[2]+"\"";   // path to be switched
       
       if (debugx)
         {
           cout << "**3** path switching command to OSP :" << endl;
           cout << par_2 << endl;
          }

         OSP=RemOpen(par_1);
         


         RemOut(OSP,"SYSTem:ERRor:ALL?");  // clear error queue
         xx=RemInp(OSP);
//         if (debugx)
//                  cout << xx << endl;

    
         RemOut(OSP,"*IDN?");

               
         xx=RemInp(OSP);
         if (debugx)
           {
            cout << endl;
            cout << "**4** remote instrument detected :" << endl;
            cout << xx << endl;    // [db_000]
            cout << endl;           
           }
    
         // clear error queue
         // in contrast to other instruments OSP
         // requires an explicite reading of the 
         // error queue until result is zero
         do 
         {
           RemOut(OSP,"SYSTem:ERRor?");  
           xx=RemInp(OSP);
         }
         while(xstoi(xx) != 0);
    
    
		 RemOut(OSP,par_2);               // switch path
		 
         RemOut(OSP,"*STB?");
         xx = RemInp(OSP);
         
         stat_STB = (int)xx[0] -0x30;    // conversion of 8-bit statusbyte

		 
         RemOut(OSP,"*ESR?");
         yy = RemInp(OSP);

         stat_ESR = (int)yy[0] -0x30;    // conversion of 8-bit error status
		 
		 if ((stat_STB == 0) && (stat_ESR == 0))
		   status_code = status_OK;
         else
           {
             if (((stat_STB & 4)==4) && ((stat_ESR & 8) ==8))
               status_code = status_INVALID_PATH;
           }
		 

         if (debugx)
         {
           RemOut(OSP,"ROUT:PATH:CAT?");
           xx = RemInp(OSP);
           cout << "**5** paths available on OSP :" << endl;
           cout << xx << endl;
           cout << endl;           
         }	 
		 

         if (debugx)
         {
          cout << "**6** OSP status byte & error status register :" << endl;
          cout << "Status STB :" ;
          cout << stat_STB << endl;
    
          cout << "Status ESR :" ;
          cout << stat_ESR << endl << endl;
         }	 

         RemClose(OSP);
 }   
else
    status_code = status_INVALID_NUMBER_OF_PARAMETERS;

// status evaluation :
if (debugx)
 if(status_code == status_OK)
   {
     cout << "**7** final status :" << endl;
     cout << "OSP path setting successfull !"  << endl;
   }
 else
   {
     switch (status_code)
     {
        case status_INVALID_ADDRESS:
             cout << "Invalid remote control address,  must include the term 'INSTR'" << endl;
             break;
                        
             
        case   status_REM_IO_ERR:
             cout << "It was not possible to access the remote control interface" << endl;
             break;

             
        case   status_INVALID_PATH:
             cout << "The specified path was not available in OSP" << endl;
             break;
      
      
        case   status_INVALID_NUMBER_OF_PARAMETERS:
             cout << "The number of parameters in the command line must be either 1,2 or 3" << endl;
             cout << "Please check the command line for ambiguous space characters" << endl;
             cout << "Examples for valid invocations : " << endl;
             cout << "SetPath_OSP.exe TCPIP::10.85.0.114::INSTR path_02 128" << endl;
             cout << "SetPath_OSP.exe TCPIP::10.85.0.114::INSTR path_02" << endl;             
             break;


        case   status_REM_OPEN_ERR:
             cout << "An error occured during opening the VISA  io-channel" << endl;
             break;

        
              
        default:
             cout << "An unknown error occured" << endl;
            
        }  // end switch
     }  // end else    

  if (debugx)
   {
    cout << endl; 
    system("PAUSE");
   }       
  return EXIT_SUCCESS;   
}




